@@ -1,4 +1,4 @@
-#include "FileZilla.h"
+#include <filezilla.h>
 #include "ControlSocket.h"
 #include "ftpcontrolsocket.h"
 #include "sftpcontrolsocket.h"
@@ -353,6 +353,8 @@ int CFileZillaEnginePrivate::List(const CListCommand &command)
 		if (pServer)
 		{
 			CServerPath path(CPathCache::Lookup(*pServer, command.GetPath(), command.GetSubDir()));
+			if (path.IsEmpty() && command.GetSubDir().IsEmpty())
+				path = command.GetPath();
 			if (!path.IsEmpty())
 			{
 				CDirectoryListing *pListing = new CDirectoryListing;
@@ -644,6 +646,7 @@ int CFileZillaEnginePrivate::ContinueConnect()
 		m_pControlSocket = new CSftpControlSocket(this);
 		break;
 	case HTTP:
+	case HTTPS:
 		m_pControlSocket = new CHttpControlSocket(this);
 		break;
 	default:
