@@ -1,10 +1,4 @@
-#include <wx/defs.h>
-#ifdef __WXMSW__
-// For AF_INET6
-#include <winsock2.h>
-#include <ws2tcpip.h>
-#endif
-#include "FileZilla.h"
+#include <filezilla.h>
 #include "transfersocket.h"
 #include "ftpcontrolsocket.h"
 #include "directorylistingparser.h"
@@ -12,9 +6,6 @@
 #include "iothread.h"
 #include "tlssocket.h"
 #include <errno.h>
-#ifndef __WXMSW__
-#include <sys/socket.h>
-#endif
 #include "proxy.h"
 #include "servercapabilities.h"
 
@@ -113,7 +104,7 @@ wxString CTransferSocket::SetupActiveTransfer(const wxString& ip)
 	}
 
 	wxString portArguments;
-	if (m_pSocketServer->GetAddressFamily() == AF_INET6)
+	if (m_pSocketServer->GetAddressFamily() == CSocket::ipv6)
 	{
 		portArguments = wxString::Format(_T("|2|%s|%d|"), ip.c_str(), port);
 	}
