@@ -1,4 +1,4 @@
-#include "FileZilla.h"
+#include <filezilla.h>
 #include "LocalTreeView.h"
 #include "queue.h"
 #include "filezillaapp.h"
@@ -198,10 +198,13 @@ protected:
 	wxDataObjectComposite* m_pDataObject;
 };
 
-IMPLEMENT_CLASS(CLocalTreeView, wxTreeCtrl)
+IMPLEMENT_CLASS(CLocalTreeView, wxTreeCtrlEx)
 
-BEGIN_EVENT_TABLE(CLocalTreeView, wxTreeCtrl)
+BEGIN_EVENT_TABLE(CLocalTreeView, wxTreeCtrlEx)
 EVT_TREE_ITEM_EXPANDING(wxID_ANY, CLocalTreeView::OnItemExpanding)
+#ifdef __WXMSW__
+EVT_TREE_SEL_CHANGING(wxID_ANY, CLocalTreeView::OnSelectionChanging)
+#endif
 EVT_TREE_SEL_CHANGED(wxID_ANY, CLocalTreeView::OnSelectionChanged)
 EVT_TREE_BEGIN_DRAG(wxID_ANY, CLocalTreeView::OnBeginDrag)
 #ifndef __WXMSW__
@@ -219,10 +222,11 @@ EVT_MENU(XRCID("ID_MKDIR"), CLocalTreeView::OnMenuMkdir)
 EVT_TREE_BEGIN_LABEL_EDIT(wxID_ANY, CLocalTreeView::OnBeginLabelEdit)
 EVT_TREE_END_LABEL_EDIT(wxID_ANY, CLocalTreeView::OnEndLabelEdit)
 EVT_CHAR(CLocalTreeView::OnChar)
+EVT_MENU(XRCID("ID_OPEN"), CLocalTreeView::OnMenuOpen)
 END_EVENT_TABLE()
 
 CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState, CQueueView *pQueueView)
-	: wxTreeCtrl(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER),
+	: wxTreeCtrlEx(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER),
 	CSystemImageList(16),
 	CStateEventHandler(pState),
 	m_pQueueView(pQueueView)
@@ -234,8 +238,6 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState,
 	pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
 	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
 
-	m_setSelection = false;
-
 	SetImageList(GetSystemImageList());
 
 #ifdef __WXMSW__
@@ -243,7 +245,12 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState,
 
 	CreateRoot();
 #else
-	AddRoot(_T("/"), GetIconIndex(dir), GetIconIndex(opened_dir));
+	wxTreeItemId root = AddRoot(_T("/"));
+	SetItemImage(root, GetIconIndex(dir), wxTreeItemIcon_Normal);
+	SetItemImage(root, GetIconIndex(opened_dir), wxTreeItemIcon_Selected);
+	SetItemImage(root, GetIconIndex(dir), wxTreeItemIcon_Expanded);
+	SetItemImage(root, GetIconIndex(opened_dir), wxTreeItemIcon_SelectedExpanded);
+
 	SetDir(_T("/"));
 #endif
 
@@ -269,9 +276,7 @@ void CLocalTreeView::SetDir(wxString localDir)
 	{
 		// TODO: UNC path, don't display it yet
 		m_currentDir = _T("");
-		m_setSelection = true;
-		SelectItem(wxTreeItemId());
-		m_setSelection = false;
+		SafeSelectItem(wxTreeItemId());
 		return;
 	}
 	m_currentDir = localDir;
@@ -279,9 +284,7 @@ void CLocalTreeView::SetDir(wxString localDir)
 #ifdef __WXMSW__
 	if (localDir == _T("\\"))
 	{
-		m_setSelection = true;
-		SelectItem(m_drives);
-		m_setSelection = false;
+		SafeSelectItem(m_drives);
 		return;
 	}
 #endif
@@ -290,9 +293,7 @@ void CLocalTreeView::SetDir(wxString localDir)
 	wxTreeItemId parent = GetNearestParent(subDirs);
 	if (!parent)
 	{
-		m_setSelection = true;
-		SelectItem(wxTreeItemId());
-		m_setSelection = false;
+		SafeSelectItem(wxTreeItemId());
 		return;
 	}
 
@@ -300,25 +301,19 @@ void CLocalTreeView::SetDir(wxString localDir)
 	{
 		wxTreeItemIdValue value;
 		wxTreeItemId child = GetFirstChild(parent, value);
-		if (child && GetItemText(child) == _T(""))
-			DisplayDir(parent, localDir);
 
-		m_setSelection = true;
-		SelectItem(parent);
-		m_setSelection = false;
-		if (parent != GetRootItem())
-			Expand(GetItemParent(parent));
+		//Not needed, item stays unexpanded
+		//if (child && GetItemText(child) == _T(""))
+		//	DisplayDir(parent, localDir);
+
+		SafeSelectItem(parent);
 		return;
 	}
 	wxTreeItemId item = MakeSubdirs(parent, localDir.Left(localDir.Length() - subDirs.Length()), subDirs);
 	if (!item)
 		return;
 
-	m_setSelection = true;
-	SelectItem(item);
-	m_setSelection = false;
-	if (item != GetRootItem())
-		Expand(GetItemParent(item));
+	SafeSelectItem(item);
 }
 
 wxTreeItemId CLocalTreeView::GetNearestParent(wxString& localDir)
@@ -479,7 +474,13 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 					return;
 
 				const wxString fullName = dirname + knownSubdir;
-				item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName), GetIconIndex(opened_dir, fullName));
+				item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName),
+#ifdef __WXMSW__
+						-1
+#else
+						GetIconIndex(opened_dir, fullName)
+#endif
+					);
 				CheckSubdirStatus(item, fullName);
 			}
 			else
@@ -528,14 +529,28 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 		else
 			matchedKnown = true;
 
-		wxTreeItemId item = AppendItem(parent, file, GetIconIndex(::dir, fullName), GetIconIndex(opened_dir, fullName));
+		wxTreeItemId item = AppendItem(parent, file, GetIconIndex(::dir, fullName),
+#ifdef __WXMSW__
+				-1
+#else
+				GetIconIndex(opened_dir, fullName)
+#endif
+			);
+
 		CheckSubdirStatus(item, fullName);
 	}
 
 	if (!matchedKnown && knownSubdir != _T(""))
 	{
 		const wxString fullName = dirname + knownSubdir;
-		wxTreeItemId item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName), GetIconIndex(opened_dir, fullName));
+		wxTreeItemId item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName),
+#ifdef __WXMSW__
+				-1
+#else
+				GetIconIndex(opened_dir, fullName)
+#endif
+			);
+
 		CheckSubdirStatus(item, fullName);
 	}
 
@@ -604,7 +619,8 @@ wxTreeItemId CLocalTreeView::MakeSubdirs(wxTreeItemId parent, wxString dirname,
 		dirname += segment + separator;
 	}
 
-	DisplayDir(parent, dirname);
+	// Not needed, stays unexpanded by default
+	// DisplayDir(parent, dirname);
 	return parent;
 }
 
@@ -826,7 +842,14 @@ void CLocalTreeView::Refresh()
 			else if (cmp < 0)
 			{
 				wxString fullname = dir.dir + *iter + separator;
-				wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(::dir, fullname), GetIconIndex(opened_dir, fullname));
+				wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(::dir, fullname),
+#ifdef __WXMSW__
+						-1
+#else
+						GetIconIndex(opened_dir, fullname)
+#endif
+					);
+
 				CheckSubdirStatus(newItem, fullname);
 				iter++;
 				inserted = true;
@@ -845,7 +868,14 @@ void CLocalTreeView::Refresh()
 		while (iter != dirs.rend())
 		{
 			wxString fullname = dir.dir + *iter + separator;
-			wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(::dir, fullname), GetIconIndex(opened_dir, fullname));
+			wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(::dir, fullname),
+#ifdef __WXMSW__
+					-1
+#else
+					GetIconIndex(opened_dir, fullname)
+#endif
+				);
+
 			CheckSubdirStatus(newItem, fullname);
 			iter++;
 			inserted = true;
@@ -897,7 +927,7 @@ int CLocalTreeView::OnCompareItems(const wxTreeItemId& item1, const wxTreeItemId
 #endif
 }
 
-void CLocalTreeView::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
+void CLocalTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
 {
 	if (notification == STATECHANGE_LOCAL_DIR)
 		SetDir(m_pState->GetLocalDir().GetPath());
@@ -1567,4 +1597,36 @@ void CLocalTreeView::RemoveDrive(wxChar drive)
 	Delete(driveItem);
 }
 
+void CLocalTreeView::OnSelectionChanging(wxTreeEvent& event)
+{
+	// On-demand open icon for selected items
+	wxTreeItemId item = event.GetItem();
+
+	if (!item)
+		return;
+
+	if (GetItemImage(item, wxTreeItemIcon_Selected) == -1)
+	{
+		int icon = GetIconIndex(opened_dir, GetDirFromItem(item));
+		SetItemImage(item, icon, wxTreeItemIcon_Selected);
+		SetItemImage(item, icon, wxTreeItemIcon_SelectedExpanded);
+	}
+}
+
 #endif
+
+void CLocalTreeView::OnMenuOpen(wxCommandEvent& event)
+{
+	if (!m_contextMenuItem.IsOk())
+		return;
+
+	wxString path = GetDirFromItem(m_contextMenuItem);
+	if (path == _T(""))
+		return;
+
+	wxString url = CState::GetAsURL(path);
+	if (url == _T(""))
+		return;
+
+	wxLaunchDefaultBrowser(url);
+}
