@@ -3,6 +3,7 @@
 
 #include "systemimagelist.h"
 #include "state.h"
+#include "treectrlex.h"
 
 class CQueueView;
 
@@ -10,7 +11,7 @@ class CQueueView;
 class CVolumeDescriptionEnumeratorThread;
 #endif
 
-class CLocalTreeView : public wxTreeCtrl, CSystemImageList, CStateEventHandler
+class CLocalTreeView : public wxTreeCtrlEx, CSystemImageList, CStateEventHandler
 {
 	DECLARE_CLASS(CLocalTreeView)
 
@@ -26,7 +27,7 @@ public:
 #endif
 
 protected:
-	virtual void OnStateChange(enum t_statechange_notifications notification, const wxString& data);
+	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
 
 	void SetDir(wxString localDir);
 	void Refresh();
@@ -54,6 +55,9 @@ protected:
 
 	DECLARE_EVENT_TABLE()
 	void OnItemExpanding(wxTreeEvent& event);
+#ifdef __WXMSW__
+	void OnSelectionChanging(wxTreeEvent& event);
+#endif
 	void OnSelectionChanged(wxTreeEvent& event);
 	void OnBeginDrag(wxTreeEvent& event);
 #ifndef __WXMSW__
@@ -70,6 +74,7 @@ protected:
 	void OnBeginLabelEdit(wxTreeEvent& event);
 	void OnEndLabelEdit(wxTreeEvent& event);
 	void OnChar(wxKeyEvent& event);
+	void OnMenuOpen(wxCommandEvent& event);
 
 #ifdef __WXMSW__
 	// React to changed drive letters
@@ -81,8 +86,6 @@ protected:
 
 	CQueueView* m_pQueueView;
 
-	bool m_setSelection;
-
 	wxTreeItemId m_contextMenuItem;
 	wxTreeItemId m_dropHighlight;
 };
