@@ -18,6 +18,7 @@ enum interfaceOptions
 	OPTION_UPDATECHECK_LASTDATE,
 	OPTION_UPDATECHECK_NEWVERSION,
 	OPTION_UPDATECHECK_CHECKBETA,
+	OPTION_UPDATECHECK_DOWNLOADDIR,
 	OPTION_DEBUG_MENU,
 	OPTION_FILEEXISTS_DOWNLOAD,
 	OPTION_FILEEXISTS_UPLOAD,
@@ -42,8 +43,6 @@ enum interfaceOptions
 	OPTION_DATE_FORMAT,
 	OPTION_SHOW_MESSAGELOG,
 	OPTION_SHOW_QUEUE,
-	OPTION_SIZE_FORMAT,
-	OPTION_SIZE_USETHOUSANDSEP,
 	OPTION_EDIT_DEFAULTEDITOR,
 	OPTION_EDIT_ALWAYSDEFAULT,
 	OPTION_EDIT_INHERITASSOCIATIONS,
@@ -60,7 +59,6 @@ enum interfaceOptions
 	OPTION_REMOTEFILELIST_COLUMN_ORDER,
 	OPTION_FILELIST_STATUSBAR,
 	OPTION_FILTERTOGGLESTATE,
-	OPTION_SIZE_DECIMALPLACES,
 	OPTION_SHOW_QUICKCONNECT,
 	OPTION_MESSAGELOG_POSITION,
 	OPTION_LAST_CONNECTED_SITE,
@@ -74,6 +72,14 @@ enum interfaceOptions
 	OPTION_COMPARE_HIDEIDENTICAL,
 	OPTION_SEARCH_SORTORDER,
 	OPTION_EDIT_TRACK_LOCAL,
+	OPTION_PREVENT_IDLESLEEP,
+	OPTION_FILTEREDIT_SIZE,
+	OPTION_INVALID_CHAR_REPLACE_ENABLE,
+	OPTION_INVALID_CHAR_REPLACE,
+	OPTION_ALREADYCONNECTED_CHOICE,
+	OPTION_EDITSTATUSDIALOG_SIZE,
+	OPTION_SPEED_DISPLAY,
+	OPTION_TOOLBAR_HIDDEN,
 
 	// Has to be last element
 	OPTIONS_NUM
