@@ -1,19 +1,21 @@
-#include "FileZilla.h"
+#include <filezilla.h>
 #include "fileexistsdlg.h"
+#include "Options.h"
+#include "sizeformatting.h"
 
 #include <wx/display.h>
-#include <wx/string.h>
 
 BEGIN_EVENT_TABLE(CFileExistsDlg, wxDialogEx)
 EVT_BUTTON(XRCID("wxID_OK"), CFileExistsDlg::OnOK)
 EVT_BUTTON(XRCID("wxID_CANCEL"), CFileExistsDlg::OnCancel)
+EVT_CHECKBOX(wxID_ANY, CFileExistsDlg::OnCheck)
 END_EVENT_TABLE()
 
 CFileExistsDlg::CFileExistsDlg(CFileExistsNotification *pNotification)
 {
 	m_pNotification = pNotification;
-	m_pAction1 = m_pAction2 = m_pAction3 = m_pAction4 = m_pAction5 = 0;
-	m_action = 0;
+	m_pAction1 = m_pAction2 = m_pAction3 = m_pAction4 = m_pAction5 = m_pAction6 = m_pAction7 = 0;
+	m_action = CFileExistsNotification::overwrite;
 	m_always = false;
 	m_queueOnly = false;
 	m_directionOnly = false;
@@ -38,16 +40,32 @@ void CFileExistsDlg::CreateControls()
 	m_pAction3 = wxDynamicCast(FindWindow(XRCID("ID_ACTION3")), wxRadioButton);
 	m_pAction4 = wxDynamicCast(FindWindow(XRCID("ID_ACTION4")), wxRadioButton);
 	m_pAction5 = wxDynamicCast(FindWindow(XRCID("ID_ACTION5")), wxRadioButton);
+	m_pAction6 = wxDynamicCast(FindWindow(XRCID("ID_ACTION6")), wxRadioButton);
+	m_pAction7 = wxDynamicCast(FindWindow(XRCID("ID_ACTION7")), wxRadioButton);
 
 	wxString localFile = m_pNotification->localFile;
 
-	wxString remoteFile = m_pNotification->remotePath.GetPath() + m_pNotification->remoteFile;
+	wxString remoteFile = m_pNotification->remotePath.FormatFilename(m_pNotification->remoteFile);
     localFile = GetPathEllipsis(localFile, FindWindow(XRCID("ID_FILE1_NAME")));
     remoteFile = GetPathEllipsis(remoteFile, FindWindow(XRCID("ID_FILE2_NAME")));
 
 	localFile.Replace(_T("&"), _T("&&"));
 	remoteFile.Replace(_T("&"), _T("&&"));
 
+	const bool thousands_separator = COptions::Get()->GetOptionVal(OPTION_SIZE_USETHOUSANDSEP) != 0;
+
+	wxString localSize;
+	if (m_pNotification->localSize != -1)
+		localSize = CSizeFormat::Format(m_pNotification->localSize, true, CSizeFormat::bytes, thousands_separator, 0);
+	else
+		localSize = _("Size unknown");
+
+	wxString remoteSize;
+	if (m_pNotification->remoteSize != -1)
+		remoteSize = CSizeFormat::Format(m_pNotification->remoteSize, true, CSizeFormat::bytes, thousands_separator, 0);
+	else
+		remoteSize = _("Size unknown");
+
 	if (m_pNotification->download)
 	{
 		wxStaticText *pStatText;
@@ -58,12 +76,7 @@ void CFileExistsDlg::CreateControls()
 
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE1_SIZE")));
 		if (pStatText)
-		{
-			if (m_pNotification->localSize != -1)
-				pStatText->SetLabel(m_pNotification->localSize.ToString() + _T(" ") + _("bytes"));
-			else
-				pStatText->SetLabel(_("Size unknown"));
-		}
+			pStatText->SetLabel(localSize);
 
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE1_TIME")));
 		if (pStatText)
@@ -82,12 +95,7 @@ void CFileExistsDlg::CreateControls()
 
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE2_SIZE")));
 		if (pStatText)
-		{
-			if (m_pNotification->remoteSize != -1)
-				pStatText->SetLabel(m_pNotification->remoteSize.ToString() + _T(" ") + _("bytes"));
-			else
-				pStatText->SetLabel(_("Size unknown"));
-		}
+			pStatText->SetLabel(remoteSize);
 
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE2_TIME")));
 		if (pStatText)
@@ -114,12 +122,7 @@ void CFileExistsDlg::CreateControls()
 
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE1_SIZE")));
 		if (pStatText)
-		{
-			if (m_pNotification->remoteSize != -1)
-				pStatText->SetLabel(m_pNotification->remoteSize.ToString() + _T(" ") + _("bytes"));
-			else
-				pStatText->SetLabel(_("Size unknown"));
-		}
+			pStatText->SetLabel(remoteSize);
 
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE1_TIME")));
 		if (pStatText)
@@ -138,12 +141,7 @@ void CFileExistsDlg::CreateControls()
 
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE2_SIZE")));
 		if (pStatText)
-		{
-			if (m_pNotification->localSize != -1)
-				pStatText->SetLabel(m_pNotification->localSize.ToString() + _T(" ") + _("bytes"));
-			else
-				pStatText->SetLabel(_("Size unknown"));
-		}
+			pStatText->SetLabel(localSize);
 
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE2_TIME")));
 		if (pStatText)
@@ -203,6 +201,9 @@ void CFileExistsDlg::LoadIcon(int id, const wxString &file)
 
 	wxFileName fn(file);
 	wxString ext = fn.GetExt();
+	if (ext == _T(""))
+		return;
+
 	wxFileType *pType = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
 	if (pType)
 	{
@@ -251,17 +252,21 @@ void CFileExistsDlg::LoadIcon(int id, const wxString &file)
 void CFileExistsDlg::OnOK(wxCommandEvent& event)
 {
 	if (m_pAction1 && m_pAction1->GetValue())
-		m_action = 0;
+		m_action = CFileExistsNotification::overwrite;
 	else if (m_pAction2 && m_pAction2->GetValue())
-		m_action = 1;
+		m_action = CFileExistsNotification::overwriteNewer;
 	else if (m_pAction3 && m_pAction3->GetValue())
-		m_action = 2;
+		m_action = CFileExistsNotification::resume;
 	else if (m_pAction4 && m_pAction4->GetValue())
-		m_action = 3;
+		m_action = CFileExistsNotification::rename;
 	else if (m_pAction5 && m_pAction5->GetValue())
-		m_action = 4;
+		m_action = CFileExistsNotification::skip;
+	else if (m_pAction6 && m_pAction6->GetValue())
+		m_action = CFileExistsNotification::overwriteSizeOrNewer;
+	else if (m_pAction7 && m_pAction7->GetValue())
+		m_action = CFileExistsNotification::overwriteSize;
 	else
-		m_action = 0;
+		m_action = CFileExistsNotification::overwrite;
 
 	m_always = XRCCTRL(*this, "ID_ALWAYS", wxCheckBox)->GetValue();
 	m_directionOnly = XRCCTRL(*this, "ID_UPDOWNONLY", wxCheckBox)->GetValue();
@@ -269,14 +274,14 @@ void CFileExistsDlg::OnOK(wxCommandEvent& event)
 	EndModal(wxID_OK);
 }
 
-int CFileExistsDlg::GetAction() const
+enum CFileExistsNotification::OverwriteAction CFileExistsDlg::GetAction() const
 {
 	return m_action;
 }
 
 void CFileExistsDlg::OnCancel(wxCommandEvent& event)
 {
-	m_action = 4;
+	m_action = CFileExistsNotification::skip;
 	EndModal(wxID_CANCEL);
 }
 
@@ -296,7 +301,7 @@ wxString CFileExistsDlg::GetPathEllipsis(wxString path, wxWindow *window)
 	wxDisplay display(wxDisplay::GetFromWindow(window));
 	wxRect rect = display.GetClientArea();
 	const int DESKTOP_WIDTH = rect.GetWidth(); // width of the desktop in pixels
-	const int maxWidth = DESKTOP_WIDTH * 0.75;
+	const int maxWidth = (int)(DESKTOP_WIDTH * 0.75);
 
 	// If the path is already short enough, don't change it
 	if (string_width <= maxWidth || path.Length() < 20)
@@ -342,3 +347,12 @@ wxString CFileExistsDlg::GetPathEllipsis(wxString path, wxWindow *window)
 
 	return left + fill + right;
 }
+
+void CFileExistsDlg::OnCheck(wxCommandEvent& event)
+{
+	if (event.GetId() != XRCID("ID_UPDOWNONLY") && event.GetId() != XRCID("ID_QUEUEONLY"))
+		return;
+
+	if (event.IsChecked())
+		XRCCTRL(*this, "ID_ALWAYS", wxCheckBox)->SetValue(true);
+}
