@@ -1,4 +1,4 @@
-#include "FileZilla.h"
+#include <filezilla.h>
 #include "state.h"
 #include "commandqueue.h"
 #include "FileZillaEngine.h"
@@ -6,32 +6,222 @@
 #include "Mainfrm.h"
 #include "queue.h"
 #include "filezillaapp.h"
-#include "RemoteListView.h"
 #include "recursive_operation.h"
 #include "statusbar.h"
 #include "local_filesys.h"
+#include "listingcomparison.h"
+
+CContextManager CContextManager::m_the_context_manager;
+
+CContextManager::CContextManager()
+{
+	m_current_context = -1;
+}
+
+CContextManager* CContextManager::Get()
+{
+	return &m_the_context_manager;
+}
+
+CState* CContextManager::CreateState(CMainFrame* pMainFrame)
+{
+	wxASSERT(pMainFrame);
+
+	CState* pState = new CState(pMainFrame);
+
+	m_contexts.push_back(pState);
+
+	NotifyHandlers(pState, STATECHANGE_NEWCONTEXT, _T(""), 0, false);
+
+	return pState;
+}
+
+void CContextManager::DestroyState(CState* pState)
+{
+	for (unsigned int i = 0; i < m_contexts.size(); i++)
+	{
+		if (m_contexts[i] != pState)
+			continue;
+
+		m_contexts.erase(m_contexts.begin() + i);
+		if ((int)i < m_current_context)
+			m_current_context--;
+		else if ((int)i == m_current_context)
+		{
+			if (i >= m_contexts.size())
+				m_current_context--;
+			NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0, false);
+		}
+
+		break;
+	}
+
+	NotifyHandlers(pState, STATECHANGE_REMOVECONTEXT, _T(""), 0, false);
+	delete pState;
+}
+
+void CContextManager::SetCurrentContext(CState* pState)
+{
+	if (GetCurrentContext() == pState)
+		return;
+
+	for (unsigned int i = 0; i < m_contexts.size(); i++)
+	{
+		if (m_contexts[i] != pState)
+			continue;
+
+		m_current_context = i;
+		NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0, false);
+	}
+}
+
+void CContextManager::DestroyAllStates()
+{
+	m_current_context = -1;
+	NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0, false);
+
+	while (!m_contexts.empty())
+	{
+		CState* pState = m_contexts.back();
+		m_contexts.pop_back();
+
+		NotifyHandlers(pState, STATECHANGE_REMOVECONTEXT, _T(""), 0, false);
+		delete pState;
+	}
+}
+
+void CContextManager::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool current_only, bool blockable)
+{
+	wxASSERT(pHandler);
+	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
+
+	std::list<t_handler> &handlers = m_handlers[notification];
+	std::list<t_handler>::const_iterator iter;
+	for (iter = handlers.begin(); iter != handlers.end(); iter++)
+	{
+		if (iter->pHandler == pHandler)
+			return;
+	}
+
+	t_handler handler;
+	handler.pHandler = pHandler;
+	handler.blockable = blockable;
+	handler.current_only = current_only;
+	handlers.push_back(handler);
+}
+
+void CContextManager::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification)
+{
+	wxASSERT(pHandler);
+	wxASSERT(notification != STATECHANGE_MAX);
+
+	if (notification == STATECHANGE_NONE)
+	{
+		for (int i = 0; i < STATECHANGE_MAX; i++)
+		{
+			std::list<t_handler> &handlers = m_handlers[i];
+			for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+			{
+				if (iter->pHandler == pHandler)
+				{
+					handlers.erase(iter);
+					break;
+				}
+			}
+		}
+	}
+	else
+	{
+		std::list<t_handler> &handlers = m_handlers[notification];
+		for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+		{
+			if (iter->pHandler == pHandler)
+			{
+				handlers.erase(iter);
+				return;
+			}
+		}
+	}
+}
+
+size_t CContextManager::HandlerCount(t_statechange_notifications notification) const
+{
+	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
+	return m_handlers[notification].size();
+}
+
+void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2, bool blocked)
+{
+	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
+
+	const std::list<t_handler> &handlers = m_handlers[notification];
+	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+	{
+		if (blocked && iter->blockable)
+			continue;
+
+		if (iter->current_only && pState != GetCurrentContext())
+			continue;
+
+		iter->pHandler->OnStateChange(pState, notification, data, data2);
+	}
+}
+
+CState* CContextManager::GetCurrentContext()
+{
+	if (m_current_context == -1)
+		return 0;
+
+	return m_contexts[m_current_context];
+}
+
+void CContextManager::NotifyAllHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
+{
+	for (unsigned int i = 0; i < m_contexts.size(); i++)
+		m_contexts[i]->NotifyHandlers(notification, data, data2);
+}
+
+void CContextManager::NotifyGlobalHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
+{
+	const std::list<t_handler> &handlers = m_handlers[notification];
+	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+		iter->pHandler->OnStateChange(0, notification, data, data2);
+}
 
 CState::CState(CMainFrame* pMainFrame)
 {
+	memset(m_blocked, 0, sizeof(m_blocked));
+
 	m_pMainFrame = pMainFrame;
 
 	m_pDirectoryListing = 0;
 	m_pServer = 0;
+	m_title = _("Not connected");
 	m_successful_connect = 0;
 
 	m_pEngine = 0;
 	m_pCommandQueue = 0;
+	m_pComparisonManager = new CComparisonManager(this);
 
 	m_pRecursiveOperation = new CRecursiveOperation(this);
 
 	m_sync_browse.is_changing = false;
 	m_sync_browse.compare = false;
+
+	m_localDir.SetPath(CLocalPath::path_separator);
+
+	m_pCertificate = 0;
+	m_pSftpEncryptionInfo = 0;
 }
 
 CState::~CState()
 {
 	delete m_pServer;
+	
+	delete m_pCertificate;
+	delete m_pSftpEncryptionInfo;
 
+	delete m_pComparisonManager;
 	delete m_pCommandQueue;
 	delete m_pEngine;
 
@@ -109,7 +299,7 @@ bool CState::SetLocalDir(const wxString& dir, wxString *error /*=0*/)
 			}
 
 			m_sync_browse.is_changing = true;
-			m_sync_browse.compare = m_pMainFrame->GetComparisonManager() && m_pMainFrame->GetComparisonManager()->IsComparing();
+			m_sync_browse.compare = m_pComparisonManager->IsComparing();
 			CListCommand *pCommand = new CListCommand(remote_path);
 			m_pCommandQueue->ProcessCommand(pCommand);
 
@@ -154,7 +344,7 @@ bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modif
 		}
 	}
 	else
-		COptions::Get()->SetOption(OPTION_LASTSERVERPATH, pDirectoryListing->path.GetSafePath());
+		m_last_path = pDirectoryListing->path;
 
 	if (m_pDirectoryListing && m_pDirectoryListing->path == pDirectoryListing->path &&
         pDirectoryListing->m_failed)
@@ -171,7 +361,7 @@ bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modif
 	else
 		NotifyHandlers(STATECHANGE_REMOTE_DIR_MODIFIED);
 
-	if (m_sync_browse.is_changing)
+	if (m_sync_browse.is_changing && !modified)
 	{
 		m_sync_browse.is_changing = false;
 		if (m_pDirectoryListing->path != m_sync_browse.remote_root && !m_pDirectoryListing->path.IsSubdirOf(m_sync_browse.remote_root, false))
@@ -210,7 +400,7 @@ bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modif
 			NotifyHandlers(STATECHANGE_LOCAL_DIR);
 
 			if (m_sync_browse.compare)
-				m_pMainFrame->GetComparisonManager()->CompareListings();
+				m_pComparisonManager->CompareListings();
 		}
 	}
 	return true;
@@ -279,7 +469,9 @@ void CState::SetServer(const CServer* server)
 {
 	if (m_pServer)
 	{
-		if (server && *server == *m_pServer)
+		if (server && *server == *m_pServer &&
+			server->GetName() == m_pServer->GetName() &&
+			server->MaximumMultipleConnections() == m_pServer->MaximumMultipleConnections())
 		{
 			// Nothing changes
 			return;
@@ -287,26 +479,33 @@ void CState::SetServer(const CServer* server)
 
 		SetRemoteDir(0);
 		delete m_pServer;
+		delete m_pCertificate;
+		m_pCertificate = 0;
+		delete m_pSftpEncryptionInfo;
+		m_pSftpEncryptionInfo = 0;
+	}
+	if (server)
+	{
+		if (m_last_server != *server)
+			m_last_path.Clear();
+		m_last_server = *server;
+
+		m_pServer = new CServer(*server);
+
+		const wxString& name = server->GetName();
+		if (!name.IsEmpty())
+			m_title = name + _T(" - ") + server->FormatServer();
+		else
+			m_title = server->FormatServer();
+	}
+	else
+	{
+		m_pServer = 0;
+		m_title = _("Not connected");
 	}
 
 	m_successful_connect = false;
 
-	CStatusBar* const pStatusBar = m_pMainFrame->GetStatusBar();
-	if (pStatusBar)
-	{
-		pStatusBar->DisplayDataType(server);
-		pStatusBar->DisplayEncrypted(server);
-	}
-
-	if (server)
-		m_pServer = new CServer(*server);
-	else
-	{
-		if (m_pServer)
-			m_pMainFrame->SetTitle(_T("FileZilla"));
-		m_pServer = 0;
-	}
-
 	NotifyHandlers(STATECHANGE_SERVER);
 }
 
@@ -315,29 +514,41 @@ const CServer* CState::GetServer() const
 	return m_pServer;
 }
 
-bool CState::Connect(const CServer& server, bool askBreak, const CServerPath& path /*=CServerPath()*/)
+wxString CState::GetTitle() const
+{
+	return m_title;
+}
+
+bool CState::Connect(const CServer& server, const CServerPath& path /*=CServerPath()*/)
 {
 	if (!m_pEngine)
 		return false;
 	if (m_pEngine->IsConnected() || m_pEngine->IsBusy() || !m_pCommandQueue->Idle())
-	{
-		if (askBreak)
-			if (wxMessageBox(_("Break current connection?"), _T("FileZilla"), wxYES_NO | wxICON_QUESTION) != wxYES)
-				return false;
 		m_pCommandQueue->Cancel();
-	}
+	m_pRecursiveOperation->StopRecursiveOperation();
+	SetSyncBrowse(false);
 
 	m_pCommandQueue->ProcessCommand(new CConnectCommand(server));
 	m_pCommandQueue->ProcessCommand(new CListCommand(path, _T(""), LIST_FLAG_FALLBACK_CURRENT));
 
-	COptions::Get()->SetLastServer(server);
-	COptions::Get()->SetOption(OPTION_LASTSERVERPATH, path.GetSafePath());
+	SetServer(&server);
 
-	const wxString& name = server.GetName();
-	if (!name.IsEmpty())
-		m_pMainFrame->SetTitle(name + _T(" - ") + server.FormatServer() + _T(" - FileZilla"));
-	else
-		m_pMainFrame->SetTitle(server.FormatServer() + _T(" - FileZilla"));
+	return true;
+}
+
+bool CState::Disconnect()
+{
+	if (!m_pEngine)
+		return false;
+
+	if (!IsRemoteConnected())
+		return true;
+	
+	if (!IsRemoteIdle())
+		return false;
+
+	SetServer(0);
+	m_pCommandQueue->ProcessCommand(new CDisconnectCommand());
 
 	return true;
 }
@@ -351,7 +562,7 @@ bool CState::CreateEngine()
 	m_pEngine = new CFileZillaEngine();
 	m_pEngine->Init(m_pMainFrame, COptions::Get());
 
-	m_pCommandQueue = new CCommandQueue(m_pEngine, m_pMainFrame);
+	m_pCommandQueue = new CCommandQueue(m_pEngine, m_pMainFrame, this);
 
 	return true;
 }
@@ -367,6 +578,9 @@ void CState::DestroyEngine()
 void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool blockable /*=true*/)
 {
 	wxASSERT(pHandler);
+	wxASSERT(pHandler->m_pState == this);
+	if (pHandler->m_pState != this)
+		return;
 	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
 
 	std::list<t_handler> &handlers = m_handlers[notification];
@@ -380,7 +594,6 @@ void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_no
 	t_handler handler;
 	handler.pHandler = pHandler;
 	handler.blockable = blockable;
-	handler.blocked = false;
 	handlers.push_back(handler);
 }
 
@@ -425,28 +638,10 @@ void CState::BlockHandlers(enum t_statechange_notifications notification)
 	if (notification == STATECHANGE_NONE)
 	{
 		for (int i = 0; i < STATECHANGE_MAX; i++)
-		{
-			std::list<t_handler> &handlers = m_handlers[i];
-			for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
-			{
-				if (!iter->blockable)
-					continue;
-
-				iter->blocked = true;
-			}
-		}
+			m_blocked[i] = true;
 	}
 	else
-	{
-		std::list<t_handler> &handlers = m_handlers[notification];
-		for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
-		{
-			if (!iter->blockable)
-					continue;
-
-			iter->blocked = true;
-		}
-	}
+		m_blocked[notification] = true;
 }
 
 void CState::UnblockHandlers(enum t_statechange_notifications notification)
@@ -456,49 +651,41 @@ void CState::UnblockHandlers(enum t_statechange_notifications notification)
 	if (notification == STATECHANGE_NONE)
 	{
 		for (int i = 0; i < STATECHANGE_MAX; i++)
-		{
-			std::list<t_handler> &handlers = m_handlers[i];
-			for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
-				iter->blocked = false;
-		}
+			m_blocked[i] = false;
 	}
 	else
-	{
-		std::list<t_handler> &handlers = m_handlers[notification];
-		for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
-			iter->blocked = false;
-	}
+		m_blocked[notification] = false;
+
 }
 
-void CState::NotifyHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/)
+void CState::NotifyHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
 	const std::list<t_handler> &handlers = m_handlers[notification];
 	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); iter++)
 	{
-		if (iter->blocked)
+		if (m_blocked[notification] && iter->blockable)
 			continue;
 
-		iter->pHandler->OnStateChange(notification, data);
+		iter->pHandler->OnStateChange(this, notification, data, data2);
 	}
+
+	CContextManager::Get()->NotifyHandlers(this, notification, data, data2, m_blocked[notification]);
 }
 
 CStateEventHandler::CStateEventHandler(CState* pState)
+	: m_pState(pState)
 {
-	wxASSERT(pState);
-
-	if (!pState)
-		return;
-
-	m_pState = pState;
 }
 
 CStateEventHandler::~CStateEventHandler()
 {
-	if (!m_pState)
-		return;
-	m_pState->UnregisterHandler(this, STATECHANGE_NONE);
+	CContextManager::Get()->UnregisterHandler(this, STATECHANGE_NONE);
+
+	const std::vector<CState*> *states = CContextManager::Get()->GetAllStates();
+	for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); iter++)
+		(*iter)->UnregisterHandler(this, STATECHANGE_NONE);
 }
 
 void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const wxString& subdir, bool queueOnly)
@@ -701,7 +888,21 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 	if (!hasDirs)
 		return true;
 
-	return m_pMainFrame->GetRemoteListView()->DownloadDroppedFiles(pRemoteDataObject, path, queueOnly);
+	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); iter++)
+	{
+		if (!iter->dir)
+			continue;
+
+		m_pRecursiveOperation->AddDirectoryToVisit(pRemoteDataObject->GetServerPath(), iter->name, path.GetPath() + CQueueView::ReplaceInvalidCharacters(iter->name), iter->link);
+	}
+
+	if (m_pComparisonManager->IsComparing())
+		m_pComparisonManager->ExitComparisonMode();
+
+	CFilterManager filter;
+	m_pRecursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, pRemoteDataObject->GetServerPath(), filter.GetActiveFilters(false));
+
+	return true;
 }
 
 bool CState::IsRemoteConnected() const
@@ -794,10 +995,7 @@ void CState::LinkIsNotDir(const CServerPath& path, const wxString& subdir)
 {
 	m_sync_browse.is_changing = false;
 
-	if (m_pRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
-		m_pRecursiveOperation->LinkIsNotDir();
-	else
-		m_pMainFrame->GetRemoteListView()->LinkIsNotDir(path, subdir);
+	NotifyHandlers(STATECHANGE_REMOTE_LINKNOTDIR, subdir, &path);
 }
 
 bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=_T("")*/, int flags /*=0*/, bool ignore_busy /*=false*/)
@@ -852,7 +1050,7 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 			else
 			{
 				m_sync_browse.is_changing = true;
-				m_sync_browse.compare = m_pMainFrame->GetRemoteListView()->IsComparing();
+				m_sync_browse.compare = m_pComparisonManager->IsComparing();
 			}
 		}
 	}
@@ -880,7 +1078,10 @@ bool CState::SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root /
 	}
 
 	if (!m_pDirectoryListing && assumed_remote_root.IsEmpty())
+	{
+		NotifyHandlers(STATECHANGE_SYNC_BROWSE);
 		return false;
+	}
 
 	m_sync_browse.is_changing = false;
 	m_sync_browse.local_root = m_localDir;
@@ -942,3 +1143,44 @@ CServerPath CState::GetSynchronizedDirectory(CLocalPath local_path)
 
 	return remote_path;
 }
+
+bool CState::RefreshRemote()
+{
+	if (!m_pCommandQueue)
+		return false;
+	
+	if (!IsRemoteConnected() || !IsRemoteIdle())
+		return false;
+
+	return ChangeRemoteDir(GetRemotePath(), _T(""), LIST_FLAG_REFRESH);
+}
+
+bool CState::GetSecurityInfo(CCertificateNotification *& pInfo)
+{
+	pInfo = m_pCertificate;
+	return m_pCertificate != 0;
+}
+
+bool CState::GetSecurityInfo(CSftpEncryptionNotification *& pInfo)
+{
+	pInfo = m_pSftpEncryptionInfo;
+	return m_pSftpEncryptionInfo != 0;
+}
+
+void CState::SetSecurityInfo(CCertificateNotification const& info)
+{
+	delete m_pCertificate;
+	m_pCertificate = 0;
+	delete m_pSftpEncryptionInfo;
+	m_pSftpEncryptionInfo = 0;
+	m_pCertificate = new CCertificateNotification(info);
+}
+
+void CState::SetSecurityInfo(CSftpEncryptionNotification const& info)
+{
+	delete m_pCertificate;
+	m_pCertificate = 0;
+	delete m_pSftpEncryptionInfo;
+	m_pSftpEncryptionInfo = 0;
+	m_pSftpEncryptionInfo = new CSftpEncryptionNotification(info);
+}
