@@ -1,4 +1,4 @@
-#include "FileZilla.h"
+#include <filezilla.h>
 #include "xmlfunctions.h"
 #include "filezillaapp.h"
 #include "Options.h"
@@ -143,9 +143,9 @@ TiXmlElement* CXmlFile::CreateEmpty()
 
 	m_pDocument = new TiXmlDocument();
 	m_pDocument->SetCondenseWhiteSpace(false);
-	m_pDocument->InsertEndChild(TiXmlDeclaration("1.0", "UTF-8", "yes"));
+	m_pDocument->LinkEndChild(new TiXmlDeclaration("1.0", "UTF-8", "yes"));
 
-	return m_pDocument->InsertEndChild(TiXmlElement("FileZilla3"))->ToElement();
+	return m_pDocument->LinkEndChild(new TiXmlElement("FileZilla3"))->ToElement();
 }
 
 char* ConvUTF8(const wxString& value)
@@ -173,16 +173,16 @@ void AddTextElement(TiXmlElement* node, const char* name, const wxString& value)
 {
 	wxASSERT(node);
 
-	TiXmlElement element(name);
-
 	char* utf8 = ConvUTF8(value);
 	if (!utf8)
 		return;
 
-    element.InsertEndChild(TiXmlText(utf8));
+	TiXmlElement *element = new TiXmlElement(name);
+
+	element->LinkEndChild(new TiXmlText(utf8));
 	delete [] utf8;
 
-	node->InsertEndChild(element);
+	node->LinkEndChild(element);
 }
 
 void AddTextElement(TiXmlElement* node, const char* name, int value)
@@ -197,11 +197,11 @@ void AddTextElementRaw(TiXmlElement* node, const char* name, const char* value)
 	wxASSERT(node);
 	wxASSERT(value && *value);
 
-	TiXmlElement element(name);
+	TiXmlElement *element = new TiXmlElement(name);
 
-    element.InsertEndChild(TiXmlText(value));
+	element->LinkEndChild(new TiXmlText(value));
 
-	node->InsertEndChild(element);
+	node->LinkEndChild(element);
 }
 
 void AddTextElement(TiXmlElement* node, const wxString& value)
@@ -221,7 +221,7 @@ void AddTextElement(TiXmlElement* node, const wxString& value)
 		break;
 	}
 
-    node->InsertEndChild(TiXmlText(utf8));
+	node->LinkEndChild(new TiXmlText(utf8));
 	delete [] utf8;
 }
 
@@ -246,7 +246,7 @@ void AddTextElementRaw(TiXmlElement* node, const char* value)
 		break;
 	}
 
-    node->InsertEndChild(TiXmlText(value));
+    node->LinkEndChild(new TiXmlText(value));
 }
 
 wxString GetTextElement_Trimmed(TiXmlElement* node, const char* name)
@@ -455,7 +455,7 @@ TiXmlElement* GetXmlFile(wxFileName file, bool create /*=true*/, wxString* error
 					*error = _("Unknown root element, the file does not appear to be generated by FileZilla.");
 				return 0;
 			}
-			pElement = pXmlDocument->InsertEndChild(TiXmlElement("FileZilla3"))->ToElement();
+			pElement = pXmlDocument->LinkEndChild(new TiXmlElement("FileZilla3"))->ToElement();
 		}
 
 		return pElement;
@@ -469,9 +469,9 @@ TiXmlElement* GetXmlFile(wxFileName file, bool create /*=true*/, wxString* error
 		// create new XML document
 		TiXmlDocument* pXmlDocument = new TiXmlDocument();
 		pXmlDocument->SetCondenseWhiteSpace(false);
-		pXmlDocument->InsertEndChild(TiXmlDeclaration("1.0", "UTF-8", "yes"));
+		pXmlDocument->LinkEndChild(new TiXmlDeclaration("1.0", "UTF-8", "yes"));
 
-		pXmlDocument->InsertEndChild(TiXmlElement("FileZilla3"));
+		pXmlDocument->LinkEndChild(new TiXmlElement("FileZilla3"));
 
 		if (!SaveXmlFile(file, pXmlDocument, 0))
 		{
@@ -740,7 +740,7 @@ void SetServer(TiXmlElement *node, const CServer& server)
 		const std::vector<wxString>& postLoginCommands = server.GetPostLoginCommands();
 		if (!postLoginCommands.empty())
 		{
-			TiXmlElement* pElement = node->InsertEndChild(TiXmlElement("PostLoginCommands"))->ToElement();
+			TiXmlElement* pElement = node->LinkEndChild(new TiXmlElement("PostLoginCommands"))->ToElement();
 			for (std::vector<wxString>::const_iterator iter = postLoginCommands.begin(); iter != postLoginCommands.end(); iter++)
 				AddTextElement(pElement, "Command", *iter);
 		}
